#include <Wire.h>
#include <Wire_IICA.h>

#include "LCD_LIB.h"


/***********************************************************************
Global variables and functions
************************************************************************/

/*---------------------------------------------------------------------
     Definition area. Write pin definition here.
----------------------------------------------------------------------*/

int swPin       = 32;                   // assign D32 pin to swPin for SW_USER.
int USERLED1    = 14;                   // assign D14 pin to USEERLED0

#define SLADDR_HDC1080   ( 0x40 )       // I2C bus slave address of HDC1080
#define MINUTE          ( 60000/16 )    // 1 minute devided by 16milli sec

unsigned int old_time = 0x0000;         // previous time(milli sec.)

unsigned char hdc1080_buff[4] =         // HDC1080 communication data area
{
    0x00,                               // high byte of Humidity
    0x00,                               // low byte of Humidity
    0x00,                               // high byte of Temp.
    0x00                                // low byte of Temp.
    
};

unsigned char humid;                    // Humidity data(unit %)
int  temp;                              // Temperature data (0.1degree unit)

// LCD display buffer aera 40characters 2lines
// display data is 16 characters/line and 2lines.
// charactor position  0123456789012345
unsigned char disp_line1[40] = " Temp. =  15.0 C";
unsigned char disp_line2[40] = " Humidity =  50%";

extern volatile unsigned char slave7;   // PCF8574 slave address start value

int count16ms = 0x0000;                 // for count 1 minute
char sw_work = 0xFF;                    // work for switch check

/******************************************************************************
LCD Function Prototypes declaration
******************************************************************************/
/* LCD intialisation function prototype */

void print_LCD(int8_t *line1,int8_t *line2);


void setup() {
  // put your setup code here, to run once:
    unsigned char work;
    unsigned char temp = 0;

    pinMode(swPin, INPUT);              // set D18pin to input mode
    pinMode(USERLED1, OUTPUT);          // set D16pin to output mode
    digitalWrite(USERLED1, 1);          // turn off LED

    Wire.setClock(100000);              // set normal mode (100kHz clock)

    Wire.begin();                       // set IICA0 for I2C bus master

/*--------------------------------------
    check PCF8574 slave address
---------------------------------------*/

    for( slave7 = 0x20 ; slave7 < 0x28 ; slave7++ )
    {
        Wire.beginTransmission(slave7); // マスタ送信で起動
        work = Wire.endTransmission(1); // 送信データ数0で終了
        delay(10);
        if ( 0x00 == work )             // スレーブからACK応答か?        
        {
            break;
        }
        
    }

    if( 0x00 != work )
    {
        for( slave7 = 0x38 ; slave7 < 0x40 ; slave7++ )
      {
        Wire.beginTransmission(slave7); // マスタ送信で起動
        work = Wire.endTransmission(1); // 送信データ数0で終了
        delay(10);
        if ( 0x00 == work )             // スレーブからACK応答か?        
        {
            break;
        }
        
      }

    }

    while ( 0x00 != work )
    {
        digitalWrite(USERLED1, temp);   // LEDを点滅
        temp ^= 1;
        delay(500);
    }


    init_LCD_S();                       // initialize LCD display

    disp_line1[14] = 0xDF;              // set degreeC character of LCD
    print_LCD( (int8_t *)disp_line1, (int8_t *)disp_line2);   // display start data

}

void loop() {
  // put your main code here, to run repeatedly:

    static char m_time = 1;
    char work;
    char sw_data;
    unsigned int time_work;
    int work_int;
    unsigned long long_work;

/*--------------------------------------
    wait for 16milli seconds interval.
----------------------------------------*/

    time_work = ( int )( millis() & 0x07FF0 ); // read milli sec data

    if ( old_time != time_work )        // check 16 milli seconds passed
    {

/*--------------------------------------
    every 16milli seconds timing
----------------------------------------*/

        old_time = time_work;           // set current time
        ++count16ms;                    // count up 16milli seconds counter

/*--------------------------------------
    switch data check timing
----------------------------------------*/

        sw_work <<= 1;                  // shift old switch data left
        sw_work += ( digitalRead(swPin) ); // read sw data
        sw_data = (sw_work & 0x03);     // extract check timing data
        if ( 0x02 == sw_data )
        {                               // sw is pushed
            m_time = 0x01;              // set measure timing flag
        }

        if ( MINUTE <= count16ms )      // check 1minute past
        {
            m_time = 0x01;              // set measure timing flag
            count16ms = 0x0000;         // clear 1 minute counter
        }

        switch ( m_time )
        {

            case 0x01:                  // senser activate timing
/*--------------------------------------
   activate HDC1080
----------------------------------------*/

                Wire.beginTransmission(SLADDR_HDC1080);  // for wake up hdc1080
                Wire.write(0x00);
                Wire.endTransmission(false);  // send HDC1080 slave address and 0x00
                m_time = 0x02;          // change measure timing flag
                break;


            case 0x02:
                m_time = 0x03;          // change measure timing flag
                break;


            case 0x03:
/*--------------------------------------
   start to read data from HDC1080
----------------------------------------*/
                Wire.requestFrom(SLADDR_HDC1080, 4, true);
                m_time = 0x04;          // change measure timing flag
                break;

            case 0x04:                  // senser read timing
/*--------------------------------------
   read received data from HS3001
----------------------------------------*/
                work = Wire.available(); // check received data
                if ( 4 != work )break;  // exit if not received

                hdc1080_buff[0] = Wire.read();// read received data
                hdc1080_buff[1] = Wire.read();// read received data
                hdc1080_buff[2] = Wire.read();// read received data
                hdc1080_buff[3] = Wire.read();// read received data

/*--------------------------------------
   Calculating Temperature
   get 0.1degreeC unit
----------------------------------------*/
                long_work =  ( hdc1080_buff[0] * 0x100UL + hdc1080_buff[1] );
                                        // merge higher 8bits and lower 6bits
                long_work *= 1650;      // multiply 10 times ofMaximum temperature
                temp = (int)((long_work>> 16 ) - 400); // adjust offset(40degreeC)

/*--------------------------------------
   Calculating Humidity
   get %
----------------------------------------*/
                long_work = ( hdc1080_buff[2] * 0x100UL + hdc1080_buff[3] );
                                        // merge higher 8bits and lower 6bits
                long_work *= 100UL;     // get percentage
                humid = (unsigned char)(long_work >> 16); // get humidity

/*--------------------------------------
   display Temperature on LCD image
----------------------------------------*/

                if (temp < 0 )
                {
                    disp_line1[9] ='-';
                    work_int = ( 0 - temp );
                }
                else
                {
                    work_int = temp;
                    disp_line1[9] =' ';     // display space
                    if ( 1000 <= work_int ) // when temperature is higher than 100 degreeC
                    {
                        disp_line1[9] ='1'; // display 1
                        work_int -= 1000;   // get digit or below 100degree
                    }
                    
                }

                disp_line1[13] = (( work_int % 10 ) + 0x30); // set 0.1degreeC digit
                work_int /= 10;     // get 1degreeC data
                disp_line1[11] = (( work_int % 10 ) + 0x30);// set 1degreeC digit
                work_int /=10;      // 10degreeC data
                disp_line1[10] = (( work_int % 10 ) + 0x30);// set 1degreeC digit

/*--------------------------------------
   set Humidity
----------------------------------------*/

                if ( 100 <= humid )
                {
                    disp_line2[12] = '1';   // set digit 100% to 1
                    disp_line2[13] = '0';   // set digit 10% to 0
                    disp_line2[14] = '0';   // set digit 1% to 0
                }
                else
                {
                    disp_line2[12] = ' ';   // set digit 100% to space
                    disp_line2[14] = (( humid % 10 ) + 0x30);   // set 1% digit
                    disp_line2[13] = (( humid / 10 ) + 0x30);   // set 10% digit
                }

                print_LCD( (int8_t *)disp_line1,(int8_t *)disp_line2);    // transmit new data to LCD display

            default:
                m_time = 0x00;          // clear measure timing flag
        }


    }

}

void print_LCD(int8_t *line1,int8_t *line2)
{
    DisplayString( 0, (int8_t *)line1 );
    DisplayString( 64, (int8_t *)line2 );
}

